import { randomUUID, createHash } from 'node:crypto';
import ms from 'ms';
import MemoryDB from '../database/MemoryDB.js';
import QRCode from '../helper/QRCode.js';
import Base64 from '../helper/Base64.js';

const getEnvValue = (envKey, defaultValue) => {
  const envValue = process.env[envKey];
  if (envValue && !envValue.includes('<>')) return envValue;
  return defaultValue;
}

const defaultExpTimeOrder = ms(getEnvValue('EXP_TIME_ORDER', '1d'));
const defaultExpTimeFinishedOrder = ms(getEnvValue('EXP_TIME_FINISHED_ORDER', '15m'));

class PagerDigital {

  normalizeOrderData(order) {
    if (order && order.products && order.products.prod) {
      if (typeof order.products.prod === 'object' && !Array.isArray(order.products.prod)) {
        order.products.prod = [order.products.prod];
      }
    }
    return order;
  }

  async startOrder(req, expirationTime = defaultExpTimeOrder) {
    const orderData = req.body;
    const minifyData = JSON.stringify(this.normalizeOrderData({...orderData, STATUS: 'STARTED'}));
    const orderHash = createHash('sha256').update(minifyData).digest('hex');
    const savedUUID = await MemoryDB.get(`order-hash:${orderHash}`);
    const orderTrackingId = savedUUID || randomUUID();
    const url = `${req.get('host')}/?orderId=${orderTrackingId}`;
    if (!savedUUID) {
      await MemoryDB.set(`order:${orderTrackingId}`, minifyData, expirationTime);
      await MemoryDB.set(`order-hash:${orderHash}`, orderTrackingId, expirationTime);
    }
    const qrCode = await QRCode.create(url);
    const imageFormat = 'svg+xml';
    return { url, orderId: orderTrackingId, qrCode: `data:image/${imageFormat};base64,${Base64.encode(qrCode)}` };
  }

  async endOrder(orderTrackingId, expirationTime = defaultExpTimeFinishedOrder) {
    const { order } = await this.findOrder(orderTrackingId);
    const minifyData = { ...order, STATUS: 'FINISHED' };
    await MemoryDB.set(`order:${orderTrackingId}`, minifyData, expirationTime);
  }

  async orderTracking(orderTrackingId) {
    const { order: { STATUS } } = await this.findOrder(orderTrackingId);
    return { status: STATUS };
  }

  async findOrder(orderTrackingId) {
    const key = `order:${orderTrackingId}`;
    const order = await MemoryDB.get(key);
    return { order: JSON.parse(order) };
  }

  async updateOrder(orderTrackingId, expirationTime = ms('15m')) {
    const { order } = await this.findOrder(orderTrackingId);
    if (order === null) {
      throw new Error('ORDER_NOT_FOUND');
    }
    const minifyData = JSON.stringify({ ...order, STATUS: 'FINISHED' });
    await MemoryDB.set(`order:${orderTrackingId}`, minifyData, expirationTime);
  }
}

export default new PagerDigital();